@@ -73,6 +73,62 @@ static __inline__ unsigned long ffnz (unsigned long word) {
 }
 #endif
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
+
+/*** 2.4.xx missing bitops, lifted from Linux ***/
+
+static inline unsigned long __ffs(unsigned long word)
+{
+        __asm__("bsfl %1,%0"
+                :"=r" (word)
+                :"rm" (word));
+        return word;
+}
+
+static inline unsigned __find_first_bit(const unsigned long *addr, unsigned size)
+{
+        unsigned x = 0;
+
+        while (x < size) {
+                unsigned long val = *addr++;
+                if (val)
+                        return __ffs(val) + x;
+                x += (sizeof(*addr)<<3);
+        }
+        return x;
+}
+
+static inline int find_next_bit(const unsigned long *addr, int size, int offset)
+{
+	const unsigned long *p = addr + (offset >> 5);
+	int set = 0, bit = offset & 31, res;
+
+	if (bit) {
+		/*
+		 * Look for nonzero in the first 32 bits:
+		 */
+		__asm__("bsfl %1,%0\n\t"
+			"jne 1f\n\t"
+			"movl $32, %0\n"
+			"1:"
+			: "=r" (set)
+			: "r" (*p >> bit));
+		if (set < (32 - bit))
+			return set + offset;
+		set = 32 - bit;
+		p++;
+	}
+	/*
+	 * No set bit yet, search remaining full words for a bit
+	 */
+	res = __find_first_bit (p, size - 32 * (p - addr));
+	return (offset + set + res);
+}
+
+#define find_first_bit(addr, size) __find_first_bit((addr), (size))
+
+#endif
+
 #if 0
 static inline unsigned long long rtai_ulldiv (unsigned long long ull,
 					      unsigned long uld,
@@ -106,7 +162,7 @@ static inline unsigned long long rtai_ulldiv (unsigned long long ull,
 #else
 
 /* do_div below taken from Linux-2.6.20 */
-
+#ifndef do_div
 #define do_div(n,base) ({ \
         unsigned long __upper, __low, __high, __mod, __base; \
         __base = (base); \
@@ -120,6 +176,7 @@ static inline unsigned long long rtai_ulldiv (unsigned long long ull,
         asm("":"=A" (n):"a" (__low),"d" (__high)); \
         __mod; \
 })
+#endif
 
 static inline unsigned long long rtai_ulldiv (unsigned long long ull, unsigned long uld, unsigned long *r)
 {
@@ -186,21 +243,23 @@ static inline long long rtai_llimd(long long ll, int mult, int div) {
 static inline unsigned long long rtai_u64div32c(unsigned long long a,
 						unsigned long b,
 						int *r) {
-    __asm__ __volatile(
-       "\n        movl    %%eax,%%ebx"
-       "\n        movl    %%edx,%%eax"
-       "\n        xorl    %%edx,%%edx"
-       "\n        divl    %%ecx"
-       "\n        xchgl   %%eax,%%ebx"
-       "\n        divl    %%ecx"
-       "\n        movl    %%edx,%%ecx"
-       "\n        movl    %%ebx,%%edx"
-       : "=a" (((unsigned long *)((void *)&a))[0]), "=d" (((unsigned long *)((void *)&a))[1])
-       : "a" (((unsigned long *)((void *)&a))[0]), "d" (((unsigned long *)((void *)&a))[1]), "c" (b)
-       : "%ebx"
-       );
 
-    return a;
+	union { unsigned long long ull; unsigned long ul[2]; } u;
+	u.ull = a;
+	__asm__ __volatile(
+	"\n        movl    %%eax,%%ebx"
+	"\n        movl    %%edx,%%eax"
+	"\n        xorl    %%edx,%%edx"
+	"\n        divl    %%ecx"
+	"\n        xchgl   %%eax,%%ebx"
+	"\n        divl    %%ecx"
+	"\n        movl    %%edx,%%ecx"
+	"\n        movl    %%ebx,%%edx"
+	: "=a" (u.ul[0]), "=d" (u.ul[1])
+	: "a"  (u.ul[0]), "d"  (u.ul[1]), "c" (b)
+	: "%ebx" );
+
+	return a;
 }
 
 #if defined(__KERNEL__) && !defined(__cplusplus)
@@ -218,11 +277,11 @@ static inline unsigned long long rtai_u64div32c(unsigned long long a,
 #include <rtai_trace.h>
 
 struct rtai_realtime_irq_s {
-        int (*handler)(unsigned irq, void *cookie);
-        void *cookie;
-        int retmode;
-        int cpumask;
-        int (*irq_ack)(unsigned int);
+	int (*handler)(unsigned irq, void *cookie);
+	void *cookie;
+	int retmode;
+	int cpumask;
+	int (*irq_ack)(unsigned int, void *);
 };
 
 /* 
@@ -448,7 +507,11 @@ extern struct rt_times rt_smp_times[RTAI_NR_CPUS];
 
 extern struct calibration_data rtai_tunables;
 
-extern volatile unsigned long rtai_cpu_lock;
+extern volatile unsigned long rtai_cpu_lock[];
+
+#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,26)
+#define apic_write_around apic_write
+#endif
 
 //#define RTAI_TASKPRI 0xf0  // simplest usage without changing Linux code base
 #if defined(CONFIG_X86_LOCAL_APIC) && defined(RTAI_TASKPRI)
@@ -568,35 +631,78 @@ static inline void rt_spin_unlock_irqrestore(unsigned long flags, spinlock_t *lo
 	rtai_restore_flags(flags);
 }
 
+#if RTAI_NR_CPUS > 2
+
+// taken from Linux, see the related code there for an explanation
+
+static inline void rtai_spin_glock(volatile unsigned long *lock)
+{
+ short inc = 0x0100;
+ __asm__ __volatile__ (
+ LOCK_PREFIX "xaddw %w0, %1\n"
+ "1:\t"
+ "cmpb %h0, %b0\n\t"
+ "je 2f\n\t"
+ "rep; nop\n\t"
+ "movb %1, %b0\n\t"
+ "jmp 1b\n"
+ "2:"
+ :"+Q" (inc), "+m" (lock[1])
+ :
+ :"memory", "cc");
+}
+
+#if defined(CONFIG_X86_OOSTORE) || defined(CONFIG_X86_PPRO_FENCE)
+# define UNLOCK_LOCK_PREFIX LOCK_PREFIX
+#else
+# define UNLOCK_LOCK_PREFIX
+#endif
+
+static inline void rtai_spin_gunlock(volatile unsigned long *lock)
+{
+ __asm__ __volatile__(
+ UNLOCK_LOCK_PREFIX "incb %0"
+ :"+m" (lock[1])
+ :
+ :"memory", "cc");
+}
+
+#else
+
+static inline void rtai_spin_glock(volatile unsigned long *lock)
+{
+	while (test_and_set_bit(31, lock)) {
+		cpu_relax();
+	}
+	barrier();
+}
+
+static inline void rtai_spin_gunlock(volatile unsigned long *lock)
+{
+	test_and_clear_bit(31, lock);
+	cpu_relax(); 
+}
+
+#endif
+
 static inline void rt_get_global_lock(void)
 {
 	barrier();
 	rtai_cli();
-	if (!test_and_set_bit(hal_processor_id(), &rtai_cpu_lock)) {
-		while (test_and_set_bit(31, &rtai_cpu_lock)) {
-			cpu_relax();
-		}
+	if (!test_and_set_bit(hal_processor_id(), &rtai_cpu_lock[0])) {
+		rtai_spin_glock(&rtai_cpu_lock[0]);
 	}
 	barrier();
 }
 
 static inline void rt_release_global_lock(void)
 {
-#if 0
 	barrier();
 	rtai_cli();
-	atomic_clear_mask((0xFFFF0001 << hal_processor_id()), (atomic_t *)&rtai_cpu_lock);
-	cpu_relax();
-	barrier();
-#else
-	barrier();
-	rtai_cli();
-	if (test_and_clear_bit(hal_processor_id(), &rtai_cpu_lock)) {
-		test_and_clear_bit(31, &rtai_cpu_lock);
-		cpu_relax();
+	if (test_and_clear_bit(hal_processor_id(), &rtai_cpu_lock[0])) {
+		rtai_spin_gunlock(&rtai_cpu_lock[0]);
 	}
 	barrier();
-#endif
 }
 
 /**
@@ -640,10 +746,8 @@ static inline int rt_global_save_flags_and_cli(void)
 
 	barrier();
 	flags = rtai_save_flags_irqbit_and_cli();
-	if (!test_and_set_bit(hal_processor_id(), &rtai_cpu_lock)) {
-		while (test_and_set_bit(31, &rtai_cpu_lock)) {
-			cpu_relax();
-		}
+	if (!test_and_set_bit(hal_processor_id(), &rtai_cpu_lock[0])) {
+		rtai_spin_glock(&rtai_cpu_lock[0]);
 		barrier();
 		return flags | 1;
 	}
@@ -662,7 +766,7 @@ static inline void rt_global_save_flags(unsigned long *flags)
 {
 	unsigned long hflags = rtai_save_flags_irqbit_and_cli();
 
-	*flags = test_bit(hal_processor_id(), &rtai_cpu_lock) ? hflags : hflags | 1;
+	*flags = test_bit(hal_processor_id(), &rtai_cpu_lock[0]) ? hflags : hflags | 1;
 	if (hflags) {
 		rtai_sti();
 	}
@@ -879,9 +983,9 @@ int rt_release_irq(unsigned irq);
 
 int ack_8259A_irq(unsigned int irq);
 
-int rt_set_irq_ack(unsigned int irq, int (*irq_ack)(unsigned int));
+int rt_set_irq_ack(unsigned int irq, int (*irq_ack)(unsigned int, void *));
 
-static inline int rt_request_irq_wack(unsigned irq, int (*handler)(unsigned irq, void *cookie), void *cookie, int retmode, int (*irq_ack)(unsigned int))
+static inline int rt_request_irq_wack(unsigned irq, int (*handler)(unsigned irq, void *cookie), void *cookie, int retmode, int (*irq_ack)(unsigned int, void *))
 {
 	int retval;
 	if ((retval = rt_request_irq(irq, handler, cookie, retmode)) < 0) {
@@ -1067,3 +1171,5 @@ do { \
 #endif /* END RTAI_TRIOSS */
 
 #endif /* !_RTAI_HAL_XN_H */
+
+
