@@ -87,7 +87,7 @@
 #define SET_ONESHOT_MODE		14
 #define SIGNAL_HANDLER	 		15
 #define TASK_USE_FPU			16
-#define GET_PRIORITIES			17  // was LINUX_USE_FPU
+#define GET_TASK_INFO			17  // was LINUX_USE_FPU
 #define HARD_TIMER_COUNT		18
 #define GET_TIME_NS			19
 #define GET_CPU_TIME_NS			20
@@ -331,6 +331,9 @@
 #define RT_SIGNAL_DISABLE	       226
 #define RT_SIGNAL_TRIGGER	       227
 
+#define SEM_RT_POLL 		       228
+#define RT_POLL_NETRPC		       229
+
 #define MAX_LXRT_FUN		       230
 
 // not recovered yet 
@@ -423,10 +426,11 @@
 #define SYNC_LINUX_SYSCALL   1
 #define ASYNC_LINUX_SYSCALL  0
 
-#include <asm/ptrace.h>
+#define SRV_NSYSCALL_REGS  7
+#define PAC_NSYSCALL_ARGS  6
 
-struct mode_regs { struct pt_regs regs; long mode; };
-struct linux_syscalls_list { long in, nr, mode; void *serv; struct mode_regs *moderegs; RT_TASK *task; void (*callback_fun)(long, long); long out, retval; };
+struct mode_regs { long regs[SRV_NSYSCALL_REGS], mode, pacargs[PAC_NSYSCALL_ARGS]; };
+struct linux_syscalls_list { int in, out, nr, mode; void *serv; struct mode_regs *moderegs; RT_TASK *task; void (*callback_fun)(long, long); long retval; };
 
 #ifdef __KERNEL__
 
@@ -510,11 +514,13 @@ int set_rt_fun_entries(struct rt_native_fun_entry *entry);
 extern "C" {
 #endif /* __cplusplus */
 
-#if CONFIG_RTAI_INTERNAL_LXRT_SUPPORT
+#if 1 // needs CONFIG_RTAI_INTERNAL_LXRT_SUPPORT no more
  
 static inline struct rt_task_struct *pid2rttask(long pid)
 {
-        return ((unsigned long)pid) > PID_MAX_LIMIT ? (struct rt_task_struct *)pid : find_task_by_pid(pid)->rtai_tskext(TSKEXT0);
+	struct task_struct *lnxtsk = find_task_by_pid(pid);
+        return lnxtsk ? lnxtsk->rtai_tskext(TSKEXT0) : NULL;
+	return ((unsigned long)pid) > PID_MAX_LIMIT ? (struct rt_task_struct *)pid : find_task_by_pid(pid)->rtai_tskext(TSKEXT0);
 }
 
 static inline long rttask2pid(struct rt_task_struct * task)
@@ -571,6 +577,8 @@ void reset_rt_fun_ext_index(struct rt_fun_entry *fun,
 
 struct apic_timer_setup_data;
 
+#ifdef CONFIG_MMU
+
 #define rt_grow_and_lock_stack(incr) \
 	do { \
 		char buf[incr]; \
@@ -578,6 +586,12 @@ struct apic_timer_setup_data;
 		mlockall(MCL_CURRENT | MCL_FUTURE); \
 	} while (0)
 
+#else
+
+#define rt_grow_and_lock_stack(incr) do { } while (0)
+
+#endif
+
 #define BIDX   0 // rt_fun_ext[0]
 #define SIZARG sizeof(arg)
 
@@ -680,6 +694,7 @@ RTAI_PROTO(int, rt_thread_join, (long thread))
 #include <unistd.h>
 #include <sys/mman.h>
 
+RTAI_PROTO(int, rt_thread_delete, (RT_TASK *task));
 static void linux_syscall_server_fun(struct linux_syscalls_list *list)
 {
 	struct linux_syscalls_list syscalls = *list;
@@ -688,28 +703,26 @@ static void linux_syscall_server_fun(struct linux_syscalls_list *list)
 	syscalls.in = syscalls.out = 0;
 
 	if ((syscalls.serv = rtai_lxrt(BIDX, sizeof(struct linux_syscalls_list), LINUX_SERVER_INIT, &syscalls).v[LOW])) {
-		struct pt_regs *regs;
-		syscalls.moderegs = (struct mode_regs *)malloc(syscalls.nr*sizeof(struct mode_regs));
-		memset(syscalls.moderegs, syscalls.nr*sizeof(struct mode_regs), 0);
+		long *regs;
+		struct mode_regs moderegs[syscalls.nr];
+		memset(moderegs, 0, sizeof(moderegs));
+		syscalls.moderegs = moderegs;
                 mlockall(MCL_CURRENT | MCL_FUTURE);
 		rtai_lxrt(BIDX, sizeof(RT_TASK *), RESUME, &syscalls.task);
-		for (;;) {
-			if (abs(rtai_lxrt(BIDX, sizeof(RT_TASK *), SUSPEND, &syscalls.serv).i[LOW]) == RTE_UNBLKD) {
-				break;
-			}
-			regs = &syscalls.moderegs[syscalls.out].regs;
-			syscalls.retval = syscall(regs->LINUX_SYSCALL_NR, regs->LINUX_SYSCALL_REG1, regs->LINUX_SYSCALL_REG2, regs->LINUX_SYSCALL_REG3, regs->LINUX_SYSCALL_REG4, regs->LINUX_SYSCALL_REG5, regs->LINUX_SYSCALL_REG6);
+		while (abs(rtai_lxrt(BIDX, sizeof(RT_TASK *), SUSPEND, &syscalls.serv).i[LOW]) < RTE_LOWERR) {
+			regs = moderegs[syscalls.out].regs;
+			syscalls.retval = syscall(regs[0], regs[1], regs[2], regs[3], regs[4], regs[5], regs[6]);
 			if (syscalls.moderegs[syscalls.out].mode == SYNC_LINUX_SYSCALL) {
 				rtai_lxrt(BIDX, sizeof(RT_TASK *), RESUME, &syscalls.task);
 			} else if (syscalls.callback_fun) {
-				syscalls.callback_fun(regs->LINUX_SYSCALL_NR, syscalls.retval);
+				syscalls.callback_fun(regs[0], syscalls.retval);
 			}
 			if (++syscalls.out >= syscalls.nr) {
 				syscalls.out = 0;
 			}
 		}
-		free(syscalls.moderegs);
         }
+	rt_thread_delete((RT_TASK *)syscalls.serv);
 }
 
 #endif /* __SUPPORT_LINUX_SERVER__ */
@@ -720,15 +733,15 @@ RTAI_PROTO(void, rt_set_linux_syscall_mode, (int mode, void (*callback_fun)(long
 	rtai_lxrt(BIDX, SIZARG, SET_LINUX_SYSCALL_MODE, &arg);
 }
 
-RTAI_PROTO(int, rt_sync_async_linux_syscall_server_create, (RT_TASK *task, int mode, void (*callback_fun)(long, long), int nr_bufd_async_calls))
+RTAI_PROTO(int, rt_create_linux_syscall_server, (RT_TASK *task, int mode, void (*callback_fun)(long, long), int nr_bufd_async_calls))
 {
 	if ((task || (task = (RT_TASK *)rtai_lxrt(BIDX, sizeof(RT_TASK *), RT_BUDDY, &task).v[LOW])) && nr_bufd_async_calls > 0) {
 		struct linux_syscalls_list syscalls;
 		syscalls.task         = task;
 		syscalls.callback_fun = callback_fun;
 		syscalls.mode         = mode;
-		syscalls.nr           = nr_bufd_async_calls;
-		if (rt_thread_create((void *)linux_syscall_server_fun, &syscalls, 0)) {
+		syscalls.nr           = nr_bufd_async_calls + 1;
+		if (rt_thread_create((void *)linux_syscall_server_fun, &syscalls, RT_THREAD_STACK_MIN + syscalls.nr*sizeof(struct mode_regs))) {
 			rtai_lxrt(BIDX, sizeof(RT_TASK *), SUSPEND, &task);
 			return 0;
 		}
@@ -736,6 +749,8 @@ RTAI_PROTO(int, rt_sync_async_linux_syscall_server_create, (RT_TASK *task, int m
 	return -1;
 }
 
+#define rt_sync_async_linux_syscall_server_create(task, mode, cbfun, nr_calls)  rt_create_linux_syscall_server(task, mode, cbfun, nr_calls)
+
 #define rt_linux_syscall_server_create(task)  rt_sync_async_linux_syscall_server_create(task, SYNC_LINUX_SYSCALL, NULL, 1);
 
 RTAI_PROTO(RT_TASK *, rt_thread_init, (unsigned long name, int priority, int max_msg_size, int policy, int cpus_allowed))
@@ -822,14 +837,14 @@ RTAI_PROTO(void,rt_make_soft_real_time,(void))
 	rtai_lxrt(BIDX, SIZARG, MAKE_SOFT_RT, &arg);
 }
 
-RTAI_PROTO(int,rt_task_delete,(RT_TASK *task))
+RTAI_PROTO(int, rt_thread_delete, (RT_TASK *task))
 {
 	struct { RT_TASK *task; } arg = { task };
 	rt_make_soft_real_time();
 	return rtai_lxrt(BIDX, SIZARG, LXRT_TASK_DELETE, &arg).i[LOW];
 }
 
-#define rt_thread_delete(task)  rt_task_delete(task)
+#define rt_task_delete(task)  rt_thread_delete(task)
 
 RTAI_PROTO(int,rt_task_yield,(void))
 {
@@ -1085,10 +1100,27 @@ RTAI_PROTO(int,rt_linux_use_fpu,(int use_fpu_flag))
 	return rtai_lxrt(BIDX, SIZARG, LINUX_USE_FPU, &arg).i[LOW];
 }
 */
+
+RTAI_PROTO(int, rt_task_get_info, (RT_TASK *task, RT_TASK_INFO *task_info))
+{
+	RT_TASK_INFO ltask_info;
+	struct { RT_TASK *task; RT_TASK_INFO *taskinfo; } arg = { task, &ltask_info };
+	if (task_info && !rtai_lxrt(BIDX, SIZARG, GET_TASK_INFO, &arg).i[LOW]) {
+		*task_info = ltask_info;
+		return 0;
+	}
+	return -EINVAL;
+}
+
 RTAI_PROTO(int, rt_get_priorities, (RT_TASK *task, int *priority, int *base_priority))
 {
-	struct { RT_TASK *task; int *priority, *base_priority; } arg = { task, priority, base_priority };
-	return rtai_lxrt(BIDX, SIZARG, GET_PRIORITIES, &arg).i[LOW];
+	RT_TASK_INFO task_info;
+	if (priority && base_priority && !rt_task_get_info(task, &task_info)) {
+		*priority      = task_info.priority;
+		*base_priority = task_info.base_priority;
+		return 0;
+	}
+	return -EINVAL;
 }
 
 RTAI_PROTO(int, rt_hard_timer_tick, (void))
@@ -1362,7 +1394,7 @@ RTAI_PROTO(int, rt_task_masked_unblock,(RT_TASK *task, unsigned long mask))
 	return rtai_lxrt(BIDX, SIZARG, WAKEUP_SLEEPING, &arg).i[LOW];
 }
 
-#define rt_task_wakeup_sleeping(task, mask)  rt_task_masked_unblock(task, RT_SCHED_DELAYED)
+#define rt_task_wakeup_sleeping(task)  rt_task_masked_unblock(task, RT_SCHED_DELAYED)
 
 RTAI_PROTO(void, rt_get_exectime, (RT_TASK *task, RTIME *exectime))
 {
@@ -1374,10 +1406,8 @@ RTAI_PROTO(void, rt_get_exectime, (RT_TASK *task, RTIME *exectime))
 
 RTAI_PROTO(void, rt_gettimeorig, (RTIME time_orig[]))
 {
-	RTIME lorig[2];
-	struct { RTIME *time_orig; } arg = { lorig };
+	struct { RTIME *time_orig; } arg = { time_orig };
 	rtai_lxrt(BIDX, SIZARG, GET_TIMEORIG, &arg);
-	memcpy(time_orig, lorig, sizeof(lorig));
 }
 
 RTAI_PROTO(RT_TASK *,ftask_init,(unsigned long name, int priority))
