@@ -36,6 +36,12 @@
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
 
+#include <linux/bitops.h>
+
+#define CONFIG_MMU 1
+
+typedef int clockid_t;
+
 #ifndef TIMER_ABSTIME
 #define TIMER_ABSTIME  0x01
 #endif
@@ -79,6 +85,14 @@ typedef int timer_t;
 typedef void irqreturn_t;
 #endif  /* LINUX_VERSION_CODE < KERNEL_VERSION(2,4,23) || __cplusplus */
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,28)
+#define msleep(delay) \
+do { \
+	set_current_state(TASK_RTAISRVSLEEP); \
+	schedule_timeout(((delay)*HZ)/1000); \
+} while(0)
+#endif
+
 #define get_tsk_addr_limit(t) ((t)->addr_limit.seg)
 
 #define task_cpu(t) ((t)->processor)
@@ -92,14 +106,31 @@ typedef void irqreturn_t;
 
 #define RTAI_LINUX_IRQ_HANDLED	/* i.e. "void" return */
 
+static inline unsigned long hweight_long(unsigned long w)
+{
+        return sizeof(w) == 4 ? hweight32(w) : hweight32(w);
+}
+
 #else /* LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0) */
 
 #define RTAI_MODULE_PARM_ARRAY(name, type, addr, size) \
 	module_param_array(name, type, addr, 0400);
 
 /* Basic class macros */
+
 #ifdef CONFIG_SYSFS
 #include <linux/device.h>
+#if  LINUX_VERSION_CODE > KERNEL_VERSION(2,6,25)
+typedef struct class class_t;
+
+#if  LINUX_VERSION_CODE > KERNEL_VERSION(2,6,26)
+#define CLASS_DEVICE_CREATE(cls, devt, device, fmt, arg...)  device_create(cls, NULL, devt, NULL, fmt, ##arg)
+#else  /* < 2.6.27 */
+#define CLASS_DEVICE_CREATE(cls, devt, device, fmt, arg...)  device_create(cls, NULL, devt, fmt, ##arg)
+#endif  /* > 2.6.26 */
+
+#define class_device_destroy(a, b)  device_destroy(a, b)
+#else
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,15)
 typedef struct class class_t;
 #define CLASS_DEVICE_CREATE(cls, devt, device, fmt, arg...) class_device_create(cls, NULL, devt, device, fmt, ## arg)
@@ -116,6 +147,7 @@ typedef struct class_simple class_t;
 #endif /* LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,13) */
 #endif /* LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,15) */
 #endif
+#endif
 
 #define mm_remap_page_range(vma,from,to,size,prot) remap_page_range(vma,from,to,size,prot)
 
@@ -141,6 +173,17 @@ typedef struct class_simple class_t;
 #define CPUMASK(name)    (name.bits[0])
 #endif /* LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,7) */
 
+#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,26)
+
+#include <linux/pid.h>
+
+#define find_task_by_pid(nr) \
+	find_task_by_pid_type_ns(PIDTYPE_PID, nr, &init_pid_ns)
+#define kill_proc(pid, sig, priv)       \
+  kill_proc_info(sig, (priv) ? SEND_SIG_PRIV : SEND_SIG_NOINFO, pid)
+
+#endif /* LINUX_VERSION_CODE <= 2.6.26 */
+
 #ifndef CONFIG_SYSFS
 typedef void * class_t;
 #define class_create(a,b)  ((void *)1)
