@@ -272,8 +272,8 @@ RTAI_SYSCALL_MODE int _rt_mbx_evdrp(MBX *mbx, void *msg, int msg_size, int space
 }
 
 
-#define CHK_MBX_MAGIC { if (mbx->magic != RT_MBX_MAGIC) \
-	{ return CONFIG_RTAI_USE_NEWERR ? RTE_OBJINV : -EINVAL; } }
+#define CHK_MBX_MAGIC \
+do { if (!mbx || mbx->magic != RT_MBX_MAGIC) return (CONFIG_RTAI_USE_NEWERR ? RTE_OBJINV : -EINVAL); } while (0)
 
 #define MBX_RET(msg_size, retval) \
 	(CONFIG_RTAI_USE_NEWERR ? retval : msg_size)
@@ -310,6 +310,13 @@ RTAI_SYSCALL_MODE int rt_typed_mbx_init(MBX *mbx, int size, int type)
 	mbx->owndby = mbx->waiting_task = NULL;
 	mbx->fbyte = mbx->lbyte = mbx->avbs = 0;
         spin_lock_init(&(mbx->lock));
+#ifdef CONFIG_RTAI_RT_POLL
+	mbx->poll_recv.pollq.prev = mbx->poll_recv.pollq.next = &(mbx->poll_recv.pollq);
+	mbx->poll_send.pollq.prev = mbx->poll_send.pollq.next = &(mbx->poll_send.pollq);
+	mbx->poll_recv.pollq.task = mbx->poll_send.pollq.task = NULL;
+        spin_lock_init(&(mbx->poll_recv.pollock));
+        spin_lock_init(&(mbx->poll_send.pollock));
+#endif
 	return 0;
 }
 
@@ -371,6 +378,8 @@ RTAI_SYSCALL_MODE int rt_mbx_delete(MBX *mbx)
 {
 	CHK_MBX_MAGIC;
 	mbx->magic = 0;
+	rt_wakeup_pollers(&mbx->poll_recv, RTE_OBJREM);
+	rt_wakeup_pollers(&mbx->poll_send, RTE_OBJREM);
 	if (rt_sem_delete(&mbx->sndsem) || rt_sem_delete(&mbx->rcvsem)) {
 		return -EFAULT;
 	}
@@ -416,12 +425,15 @@ RTAI_SYSCALL_MODE int _rt_mbx_send(MBX *mbx, void *msg, int msg_size, int space)
 	while (msg_size) {
 		if ((retval = mbx_wait(mbx, &mbx->frbs, rt_current))) {
 			rt_sem_signal(&mbx->sndsem);
-			return MBX_RET(msg_size, retval);
+			retval = MBX_RET(msg_size, retval);
+			rt_wakeup_pollers(&mbx->poll_recv, retval);
+			return retval;
 		}
 		msg_size = mbxput(mbx, (char **)(&msg), msg_size, space);
 		mbx_signal(mbx);
 	}
 	rt_sem_signal(&mbx->sndsem);
+	rt_wakeup_pollers(&mbx->poll_recv, 0);
 	return 0;
 }
 
@@ -446,6 +458,7 @@ RTAI_SYSCALL_MODE int _rt_mbx_send_wp(MBX *mbx, void *msg, int msg_size, int spa
 {
 	unsigned long flags;
 	RT_TASK *rt_current = RT_CURRENT;
+	int size = msg_size;
 
 	CHK_MBX_MAGIC;
 	flags = rt_global_save_flags_and_cli();
@@ -462,6 +475,9 @@ RTAI_SYSCALL_MODE int _rt_mbx_send_wp(MBX *mbx, void *msg, int msg_size, int spa
 	} else {
 		rt_global_restore_flags(flags);
 	}
+	if (msg_size < size) {
+		rt_wakeup_pollers(&mbx->poll_recv, 0);
+	}
 	return msg_size;
 }
 
@@ -496,6 +512,7 @@ RTAI_SYSCALL_MODE int _rt_mbx_send_if(MBX *mbx, void *msg, int msg_size, int spa
 		mbxput(mbx, (char **)(&msg), msg_size, space);
 		mbx_signal(mbx);
 		rt_sem_signal(&mbx->sndsem);
+		rt_wakeup_pollers(&mbx->poll_recv, 0);
 		return 0;
 	}
 	rt_global_restore_flags(flags);
@@ -538,12 +555,15 @@ RTAI_SYSCALL_MODE int _rt_mbx_send_until(MBX *mbx, void *msg, int msg_size, RTIM
 	while (msg_size) {
 		if ((retval = mbx_wait_until(mbx, &mbx->frbs, time, rt_current))) {
 			rt_sem_signal(&mbx->sndsem);
-			return MBX_RET(msg_size, retval);
+			retval = MBX_RET(msg_size, retval);
+			rt_wakeup_pollers(&mbx->poll_recv, retval);
+			return retval;
 		}
 		msg_size = mbxput(mbx, (char **)(&msg), msg_size, space);
 		mbx_signal(mbx);
 	}
 	rt_sem_signal(&mbx->sndsem);
+	rt_wakeup_pollers(&mbx->poll_recv, 0);
 	return 0;
 }
 
@@ -608,12 +628,16 @@ RTAI_SYSCALL_MODE int _rt_mbx_receive(MBX *mbx, void *msg, int msg_size, int spa
 	while (msg_size) {
 		if ((retval = mbx_wait(mbx, &mbx->avbs, rt_current))) {
 			rt_sem_signal(&mbx->rcvsem);
+			retval = MBX_RET(msg_size, retval);
+			rt_wakeup_pollers(&mbx->poll_recv, retval);
+			return retval;
 			return MBX_RET(msg_size, retval);
 		}
 		msg_size = mbxget(mbx, (char **)(&msg), msg_size, space);
 		mbx_signal(mbx);
 	}
 	rt_sem_signal(&mbx->rcvsem);
+	rt_wakeup_pollers(&mbx->poll_send, 0);
 	return 0;
 }
 
@@ -639,6 +663,7 @@ RTAI_SYSCALL_MODE int _rt_mbx_receive_wp(MBX *mbx, void *msg, int msg_size, int
 {
 	unsigned long flags;
 	RT_TASK *rt_current = RT_CURRENT;
+	int size = msg_size;
 
 	CHK_MBX_MAGIC;
 	flags = rt_global_save_flags_and_cli();
@@ -655,6 +680,9 @@ RTAI_SYSCALL_MODE int _rt_mbx_receive_wp(MBX *mbx, void *msg, int msg_size, int
 	} else {
 		rt_global_restore_flags(flags);
 	}
+	if (msg_size < size) {
+		rt_wakeup_pollers(&mbx->poll_send, 0);
+	}
 	return msg_size;
 }
 
@@ -694,6 +722,7 @@ RTAI_SYSCALL_MODE int _rt_mbx_receive_if(MBX *mbx, void *msg, int msg_size, int
 		mbxget(mbx, (char **)(&msg), msg_size, space);
 		mbx_signal(mbx);
 		rt_sem_signal(&mbx->rcvsem);
+		rt_wakeup_pollers(&mbx->poll_send, 0);
 		return 0;
 	}
 	rt_global_restore_flags(flags);
@@ -736,12 +765,15 @@ RTAI_SYSCALL_MODE int _rt_mbx_receive_until(MBX *mbx, void *msg, int msg_size, R
 	while (msg_size) {
 		if ((retval = mbx_wait_until(mbx, &mbx->avbs, time, rt_current))) {
 			rt_sem_signal(&mbx->rcvsem);
-			return MBX_RET(msg_size, retval);
+			retval = MBX_RET(msg_size, retval);
+			rt_wakeup_pollers(&mbx->poll_recv, retval);
+			return retval;
 		}
 		msg_size = mbxget(mbx, (char **)(&msg), msg_size, space);
 		mbx_signal(mbx);
 	}
 	rt_sem_signal(&mbx->rcvsem);
+	rt_wakeup_pollers(&mbx->poll_send, 0);
 	return 0;
 }
 
@@ -918,13 +950,20 @@ struct rt_native_fun_entry rt_mbx_entries[] = {
 extern int set_rt_fun_entries(struct rt_native_fun_entry *entry);
 extern void reset_rt_fun_entries(struct rt_native_fun_entry *entry);
 
+static int recv_blocks(void *mbx) { return ((MBX *)mbx)->avbs <= 0; }
+static int send_blocks(void *mbx) { return ((MBX *)mbx)->frbs <= 0; }
+
 int __rtai_mbx_init (void)
 {
+	rt_poll_ofstfun[RT_POLL_MBX_RECV].topoll = recv_blocks;
+	rt_poll_ofstfun[RT_POLL_MBX_SEND].topoll = send_blocks;
 	return set_rt_fun_entries(rt_mbx_entries);
 }
 
 void __rtai_mbx_exit (void)
 {
+	rt_poll_ofstfun[RT_POLL_MBX_RECV].topoll = NULL;
+	rt_poll_ofstfun[RT_POLL_MBX_SEND].topoll = NULL;
 	reset_rt_fun_entries(rt_mbx_entries);
 }
 
