@@ -69,6 +69,14 @@ do { \
 	task = (void *)(long)(CONFIG_RTAI_USE_NEWERR ? (((void *)rt_current->blocked_on != RTP_UNBLKD) ? RTE_TIMOUT : RTE_UNBLKD) : 0); \
 } while (0)
 
+/* ++++++++++++++++++++++++++ TASK POINTERS CHECKS +++++++++++++++++++++++++ */
+
+#define CHECK_SENDER_MAGIC(task) \
+do { if (task->magic != RT_TASK_MAGIC) return TASK_INVAL; } while (0)
+
+#define CHECK_RECEIVER_MAGIC(task) \
+do { if (task && task->magic != RT_TASK_MAGIC) return TASK_INVAL; } while (0)
+
 /* +++++++++++++++++++++++++++++ ASYNC SENDS ++++++++++++++++++++++++++++++++ */
 
 /**
@@ -106,9 +114,7 @@ RTAI_SYSCALL_MODE RT_TASK *rt_send(RT_TASK *task, unsigned long msg)
 	DECLARE_RT_CURRENT;
 	unsigned long flags;
 
-	if (task->magic != RT_TASK_MAGIC) {
-		return TASK_INVAL;
-	}
+	CHECK_SENDER_MAGIC(task);
 
 	flags = rt_global_save_flags_and_cli();
 	ASSIGN_RT_CURRENT;
@@ -172,9 +178,7 @@ RTAI_SYSCALL_MODE RT_TASK *rt_send_if(RT_TASK *task, unsigned long msg)
 	DECLARE_RT_CURRENT;
 	unsigned long flags;
 
-	if (task->magic != RT_TASK_MAGIC) {
-		return TASK_INVAL;
-	}
+	CHECK_SENDER_MAGIC(task);
 
 	flags = rt_global_save_flags_and_cli();
 	ASSIGN_RT_CURRENT;
@@ -244,9 +248,7 @@ RTAI_SYSCALL_MODE RT_TASK *rt_send_until(RT_TASK *task, unsigned long msg, RTIME
 	DECLARE_RT_CURRENT;
 	unsigned long flags;
 
-	if (task->magic != RT_TASK_MAGIC) {
-		return TASK_INVAL;
-	}
+	CHECK_SENDER_MAGIC(task);
 
 	flags = rt_global_save_flags_and_cli();
 	ASSIGN_RT_CURRENT;
@@ -383,14 +385,12 @@ RTAI_SYSCALL_MODE RT_TASK *rt_rpc(RT_TASK *task, unsigned long to_do, void *resu
 	DECLARE_RT_CURRENT;
 	unsigned long flags;
 
-	if (task->magic != RT_TASK_MAGIC) {
-		return TASK_INVAL;
-	}
+	CHECK_SENDER_MAGIC(task);
 
 	flags = rt_global_save_flags_and_cli();
 	ASSIGN_RT_CURRENT;
 	if ((task->state & RT_SCHED_RECEIVE) &&
-		(!task->msg_queue.task || task->msg_queue.task == rt_current)) {
+	    (!task->msg_queue.task || task->msg_queue.task == rt_current)) {
 		rt_current->msg = task->msg = to_do;
 		task->msg_queue.task = rt_current;
 		task->ret_queue.task = NULL;
@@ -464,13 +464,12 @@ RTAI_SYSCALL_MODE RT_TASK *rt_rpc_if(RT_TASK *task, unsigned long to_do, void *r
 	DECLARE_RT_CURRENT;
 	unsigned long flags;
 
-	if (task->magic != RT_TASK_MAGIC) {
-		return TASK_INVAL;
-	}
+	CHECK_SENDER_MAGIC(task);
+
 	flags = rt_global_save_flags_and_cli();
 	ASSIGN_RT_CURRENT;
 	if ((task->state & RT_SCHED_RECEIVE) &&
-	      (!task->msg_queue.task || task->msg_queue.task == rt_current)) {
+	    (!task->msg_queue.task || task->msg_queue.task == rt_current)) {
 		rt_current->msg = task->msg = to_do;
 		task->msg_queue.task = rt_current;
 		task->ret_queue.task = NULL;
@@ -541,9 +540,7 @@ RTAI_SYSCALL_MODE RT_TASK *rt_rpc_until(RT_TASK *task, unsigned long to_do, void
 	DECLARE_RT_CURRENT;
 	unsigned long flags;
 
-	if (task->magic != RT_TASK_MAGIC) {
-		return TASK_INVAL;
-	}
+	CHECK_SENDER_MAGIC(task);
 
 	flags = rt_global_save_flags_and_cli();
 	ASSIGN_RT_CURRENT;
@@ -560,12 +557,13 @@ RTAI_SYSCALL_MODE RT_TASK *rt_rpc_until(RT_TASK *task, unsigned long to_do, void
 		if (task->state != RT_SCHED_READY && (task->state &= ~(RT_SCHED_RECEIVE | RT_SCHED_DELAYED)) == RT_SCHED_READY) {
 			enq_ready_task(task);
 		}
+		enqueue_blocked(rt_current, &task->ret_queue, 0);
 		rt_current->state |= (RT_SCHED_RETURN | RT_SCHED_DELAYED);
 	} else {
 		rt_current->msg = to_do;
+		enqueue_blocked(rt_current, &task->msg_queue, 0);
 		rt_current->state |= (RT_SCHED_RPC | RT_SCHED_DELAYED);
 	}
-	enqueue_blocked(rt_current, &task->ret_queue, 0);
 	enqueue_resqtsk(task);
 	pass_prio(task, rt_current);
 	rem_ready_current(rt_current);
@@ -697,9 +695,7 @@ RTAI_SYSCALL_MODE RT_TASK *rt_return(RT_TASK *task, unsigned long result)
 	DECLARE_RT_CURRENT;
 	unsigned long flags;
 
-	if (task->magic != RT_TASK_MAGIC) {
-		return TASK_INVAL;
-	}
+	CHECK_SENDER_MAGIC(task);
 
 	flags = rt_global_save_flags_and_cli();
 	ASSIGN_RT_CURRENT;
@@ -761,9 +757,7 @@ RTAI_SYSCALL_MODE RT_TASK *rt_evdrp(RT_TASK *task, void *msg)
 {
 	DECLARE_RT_CURRENT;
 
-	if (task && task->magic != RT_TASK_MAGIC) {
-		return TASK_INVAL;
-	}
+	CHECK_RECEIVER_MAGIC(task);
 
 	ASSIGN_RT_CURRENT;
 	if (!task) task = (rt_current->msg_queue.next)->task;
@@ -816,9 +810,7 @@ RTAI_SYSCALL_MODE RT_TASK *rt_receive(RT_TASK *task, void *msg)
 	DECLARE_RT_CURRENT;
 	unsigned long flags;
 
-	if (task && task->magic != RT_TASK_MAGIC) {
-		return TASK_INVAL;
-	}
+	CHECK_RECEIVER_MAGIC(task);
 
 	flags = rt_global_save_flags_and_cli();
 	ASSIGN_RT_CURRENT;
@@ -904,9 +896,7 @@ RTAI_SYSCALL_MODE RT_TASK *rt_receive_if(RT_TASK *task, void *msg)
 	DECLARE_RT_CURRENT;
 	unsigned long flags;
 
-	if (task && task->magic != RT_TASK_MAGIC) {
-		return TASK_INVAL;
-	}
+	CHECK_RECEIVER_MAGIC(task);
 
 	flags = rt_global_save_flags_and_cli();
 	ASSIGN_RT_CURRENT;
@@ -998,9 +988,7 @@ RTAI_SYSCALL_MODE RT_TASK *rt_receive_until(RT_TASK *task, void *msg, RTIME time
 	DECLARE_RT_CURRENT;
 	unsigned long flags;
 
-	if (task && task->magic != RT_TASK_MAGIC) {
-		return TASK_INVAL;
-	}
+	CHECK_RECEIVER_MAGIC(task);
 
 	flags = rt_global_save_flags_and_cli();
 	ASSIGN_RT_CURRENT;
