@@ -23,8 +23,10 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+
 #include <linux/module.h>
 #include <asm/uaccess.h>
+#include <asm/unistd.h>
 
 #include <rtai_schedcore.h>
 #include <rtai_prinher.h>
@@ -100,7 +102,7 @@ int rt_get_inher_prio(RT_TASK *task)
 	if (task->magic != RT_TASK_MAGIC) {
 		return -EINVAL;
 	}
-	return task->base_priority;
+	return task->priority;
 }
 
 
@@ -108,7 +110,7 @@ int rt_get_inher_prio(RT_TASK *task)
  * @anchor rt_get_priorities
  * @brief Check inheredited and base priority.
  * 
- * rt_task_get_priorities returns the base and inherited priorities of a task.
+ * rt_get_priorities returns the base and inherited priorities of a task.
  *
  * Recall that a task has a base native priority, assigned at its
  * birth or by @ref rt_change_prio(), and an actual, inherited,
@@ -120,10 +122,11 @@ int rt_get_inher_prio(RT_TASK *task)
  *
  * @param base_priority the base priority.
  *
- * @return rt_task_get_priority returns 0 if non NULL priority addresses
+ * @return rt_get_priority returns 0 if non NULL priority addresses
  * are given, EINVAL if addresses are NULL or task is not a valid object.
  *
  */
+
 RTAI_SYSCALL_MODE int rt_get_priorities(RT_TASK *task, int *priority, int *base_priority)
 {
 	if (!task) {
@@ -132,13 +135,35 @@ RTAI_SYSCALL_MODE int rt_get_priorities(RT_TASK *task, int *priority, int *base_
 	if (task->magic != RT_TASK_MAGIC || !priority || !base_priority) {
 		return -EINVAL;
 	}
-	if ((unsigned long)priority < PAGE_OFFSET) {
-		rt_put_user(task->priority, priority);
-		rt_put_user(task->base_priority, base_priority);
-	} else {
-		*priority      = task->priority;
-		*base_priority = task->base_priority;
+	*priority      = task->priority;
+	*base_priority = task->base_priority;
+	return 0;
+}
+
+/**
+ * @anchor rt_task_get_info
+ * @brief Get task task data listed in RT_TASK_INFO type.
+ * 
+ * rt_task_get_info returns task data listed in RT_TASK_INFO type.
+ *
+ * @param task is the task of interest, NULL can be used for the current task.
+ * @param task_info a pointer to RT_TASK_INFO.
+ *
+ * @return -EINVAL if task is not valid or task_info is NULL, 0 if OK.
+ *
+ */
+
+RTAI_SYSCALL_MODE int rt_task_get_info(RT_TASK *task, RT_TASK_INFO *task_info)
+{
+	if (!task) {
+		task = RT_CURRENT;
+	} 
+	if (task->magic != RT_TASK_MAGIC || task_info == NULL) {
+		return -EINVAL;
 	}
+	task_info->period        = task->period;
+	task_info->base_priority = task->base_priority;
+	task_info->priority      = task->priority;
 	return 0;
 }
 
@@ -166,6 +191,7 @@ RTAI_SYSCALL_MODE int rt_change_prio(RT_TASK *task, int priority)
 {
 	unsigned long flags;
 	int prio, base_priority;
+	RT_TASK *rhead;
 
 	if (task->magic != RT_TASK_MAGIC || priority < 0) {
 		return -EINVAL;
@@ -185,10 +211,11 @@ RTAI_SYSCALL_MODE int rt_change_prio(RT_TASK *task, int priority)
 			task->priority = priority;
 			if (task->state == RT_SCHED_READY) {
 				if ((task->rprev)->priority > task->priority || (task->rnext)->priority < task->priority) {
+					rhead = rt_smp_linux_task[task->runnable_on_cpus].rnext;
 					(task->rprev)->rnext = task->rnext;
 					(task->rnext)->rprev = task->rprev;
 					enq_ready_task(task);
-					if (task == rt_smp_linux_task[task->runnable_on_cpus].rnext) {
+					if (rhead != rt_smp_linux_task[task->runnable_on_cpus].rnext) {
 #ifdef CONFIG_SMP
 						__set_bit(task->runnable_on_cpus & 0x1F, &schedmap);
 #else
@@ -247,7 +274,6 @@ RT_TASK *rt_whoami(void)
 }
 
 
-
 /**
  * @anchor rt_task_yield
  * Yield the current task.
@@ -255,6 +281,8 @@ RT_TASK *rt_whoami(void)
  * @ref rt_task_yield() stops the current task and takes it at the end
  * of the list of ready tasks having its same priority. The scheduler
  * makes the next ready task of the same priority active.
+ * If the current task has the highest priority no more then it results
+ * in an immediate rescheduling.
  *
  * Recall that RTAI schedulers allow only higher priority tasks to
  * preempt the execution of lower priority ones. So equal priority
@@ -271,15 +299,20 @@ void rt_task_yield(void)
 	unsigned long flags;
 
 	flags = rt_global_save_flags_and_cli();
-	task = (rt_current = RT_CURRENT)->rnext;
-	while (rt_current->priority == task->priority) {
-		task = task->rnext;
-	}
-	if (task != rt_current->rnext) {
-		(rt_current->rprev)->rnext = rt_current->rnext;
-		(rt_current->rnext)->rprev = rt_current->rprev;
-		task->rprev = (rt_current->rprev = task->rprev)->rnext = rt_current;
-		rt_current->rnext = task;
+	rt_current = RT_CURRENT;
+	if (rt_smp_linux_task[rt_current->runnable_on_cpus].rnext == rt_current) {
+		task = rt_current->rnext;
+		while (rt_current->priority == task->priority) {
+			task = task->rnext;
+		}
+		if (task != rt_current->rnext) {
+			(rt_current->rprev)->rnext = rt_current->rnext;
+			(rt_current->rnext)->rprev = rt_current->rprev;
+			task->rprev = (rt_current->rprev = task->rprev)->rnext = rt_current;
+			rt_current->rnext = task;
+			rt_schedule();
+		}
+	} else {
 		rt_schedule();
 	}
 	rt_global_restore_flags(flags);
@@ -689,7 +722,7 @@ void rt_gettimeorig(RTIME time_orig[])
  */
 RTAI_SYSCALL_MODE int rt_task_make_periodic_relative_ns(RT_TASK *task, RTIME start_delay, RTIME period)
 {
-	long flags;
+	unsigned long flags;
 
 	if (!task) {
 		task = RT_CURRENT;
@@ -748,7 +781,7 @@ RTAI_SYSCALL_MODE int rt_task_make_periodic_relative_ns(RT_TASK *task, RTIME sta
  */
 RTAI_SYSCALL_MODE int rt_task_make_periodic(RT_TASK *task, RTIME start_time, RTIME period)
 {
-	long flags;
+	unsigned long flags;
 
 	if (!task) {
 		task = RT_CURRENT;
@@ -793,7 +826,7 @@ RTAI_SYSCALL_MODE int rt_task_make_periodic(RT_TASK *task, RTIME start_time, RTI
 int rt_task_wait_period(void)
 {
 	DECLARE_RT_CURRENT;
-	long flags;
+	unsigned long flags;
 
 	flags = rt_global_save_flags_and_cli();
 	ASSIGN_RT_CURRENT;
@@ -815,7 +848,15 @@ int rt_task_wait_period(void)
 		rt_schedule();
 		blocked_on = rt_current->blocked_on;
 		rt_global_restore_flags(flags);
+#ifdef CONFIG_M68K
+		//Workaround of a gcc bug
+		if(blocked_on == RTP_OBJREM) {
+			__asm__ __volatile__ ("nop");
+		}
+		return likely(!blocked_on) ? 0L : RTE_UNBLKD;
+#else
 		return likely(!blocked_on) ? 0 : RTE_UNBLKD;
+#endif
 	}
 	rt_global_restore_flags(flags);
 	return RTE_TMROVRN;
@@ -824,7 +865,7 @@ int rt_task_wait_period(void)
 RTAI_SYSCALL_MODE void rt_task_set_resume_end_times(RTIME resume, RTIME end)
 {
 	RT_TASK *rt_current;
-	long flags;
+	unsigned long flags;
 
 	flags = rt_global_save_flags_and_cli();
 	rt_current = RT_CURRENT;
@@ -849,7 +890,7 @@ RTAI_SYSCALL_MODE void rt_task_set_resume_end_times(RTIME resume, RTIME end)
 
 RTAI_SYSCALL_MODE int rt_set_resume_time(RT_TASK *task, RTIME new_resume_time)
 {
-	long flags;
+	unsigned long flags;
 
 	if (task->magic != RT_TASK_MAGIC) {
 		return -EINVAL;
@@ -870,7 +911,7 @@ RTAI_SYSCALL_MODE int rt_set_resume_time(RT_TASK *task, RTIME new_resume_time)
 
 RTAI_SYSCALL_MODE int rt_set_period(RT_TASK *task, RTIME new_period)
 {
-	long flags;
+	unsigned long flags;
 
 	if (task->magic != RT_TASK_MAGIC) {
 		return -EINVAL;
@@ -1030,11 +1071,12 @@ RTAI_SYSCALL_MODE int rt_task_masked_unblock(RT_TASK *task, unsigned long mask)
 			rem_timed_task(task);
 		}
 		if (task->state != RT_SCHED_READY && (task->state &= ~mask) == RT_SCHED_READY) {
+			task->blocked_on = RTP_UNBLKD;
 			enq_ready_task(task);
 			RT_SCHEDULE(task, rtai_cpuid());
 		}
 		rt_global_restore_flags(flags);
-		return (int)((unsigned long)(task->blocked_on = RTP_UNBLKD));
+		return RTE_UNBLKD;
 	}
 	return 0;
 }
@@ -1114,7 +1156,7 @@ void rt_dequeue_blocked(RT_TASK *task)
 
 int rt_renq_current(RT_TASK *rt_current, int priority)
 {
-	return renq_current(rt_current, priority);
+	return renq_ready_task(rt_current, priority);
 }
 
 /* ++++++++++++++++++++++++ NAMED TASK INIT/DELETE ++++++++++++++++++++++++++ */
@@ -1867,6 +1909,8 @@ void krtai_objects_release(void)
 
 /* +++++++++++++++++++++++++ SUPPORT FOR IRQ TASKS ++++++++++++++++++++++++++ */
 
+#ifdef CONFIG_RTAI_USI
+
 #include <rtai_tasklets.h>
 
 extern struct rtai_realtime_irq_s rtai_realtime_irq[];
@@ -1943,6 +1987,8 @@ RTAI_SYSCALL_MODE void usp_request_rtc(int rtc_freq, void *handler)
 		
 }
 
+#endif
+
 /* +++++++++++++++++ SUPPORT FOR THE LINUX SYSCALL SERVER +++++++++++++++++++ */
 
 RTAI_SYSCALL_MODE void rt_set_linux_syscall_mode(long mode, void (*callback_fun)(long, long))
@@ -1953,20 +1999,50 @@ RTAI_SYSCALL_MODE void rt_set_linux_syscall_mode(long mode, void (*callback_fun)
 
 void rt_exec_linux_syscall(RT_TASK *rt_current, struct linux_syscalls_list *syscalls, struct pt_regs *regs)
 {
-	struct { long in, nr, mode; RT_TASK *serv; struct mode_regs *moderegs; } from;
+	int in, sz;
+	struct mode_regs moderegs;
+	struct { int in, out, nr, mode; RT_TASK *serv; } from;
 
 	rt_copy_from_user(&from, syscalls, sizeof(from));
-	from.serv->priority = rt_current->priority + BASE_SOFT_PRIORITY;
-	rt_put_user(from.mode, &from.moderegs[from.in].mode);
-	rt_copy_to_user(&from.moderegs[from.in].regs, regs, sizeof(struct pt_regs));
+	in = from.in;
 	if (++from.in >= from.nr) {
 		from.in = 0;
 	}
+	if (from.mode == ASYNC_LINUX_SYSCALL && from.in == from.out) {
+		regs->LINUX_SYSCALL_RETREG = -1;
+		return;
+	}
+
+#if defined( __NR_socketcall)
+	if (regs->LINUX_SYSCALL_NR == __NR_socketcall) {
+		memcpy(moderegs.pacargs, (void *)regs->LINUX_SYSCALL_REG2, sizeof(moderegs.pacargs));
+		moderegs.regs[2] = (long)(&syscalls->moderegs[in].pacargs);
+		sz = sizeof(moderegs);
+	} else
+#endif
+	{
+		moderegs.regs[2] = regs->LINUX_SYSCALL_REG2;
+		sz = offsetof(struct mode_regs, pacargs);
+	}
+
+	moderegs.regs[0] = regs->LINUX_SYSCALL_NR;
+	moderegs.regs[1] = regs->LINUX_SYSCALL_REG1;
+	moderegs.regs[3] = regs->LINUX_SYSCALL_REG3;
+	moderegs.regs[4] = regs->LINUX_SYSCALL_REG4;
+	moderegs.regs[5] = regs->LINUX_SYSCALL_REG5;
+	moderegs.regs[6] = regs->LINUX_SYSCALL_REG6;
+	moderegs.mode = from.mode;
+	rt_copy_to_user(&syscalls->moderegs[in].regs, &moderegs, sz);
 	rt_put_user(from.in, &syscalls->in);
-	rt_task_resume(from.serv);
+	if (from.serv->suspdepth >= -from.nr) {
+		from.serv->priority = rt_current->priority + BASE_SOFT_PRIORITY;
+		rt_task_resume(from.serv);
+	}
 	if (from.mode == SYNC_LINUX_SYSCALL) {
 		rt_task_suspend(rt_current);
 		rt_get_user(regs->LINUX_SYSCALL_RETREG, &syscalls->retval);
+	} else {
+		regs->LINUX_SYSCALL_RETREG = -EINPROGRESS;
 	}
 }
 
@@ -2128,6 +2204,8 @@ EXPORT_SYMBOL(rt_get_time_cpuid);
 EXPORT_SYMBOL(rt_get_time_ns);
 EXPORT_SYMBOL(rt_get_time_ns_cpuid);
 EXPORT_SYMBOL(rt_get_cpu_time_ns);
+EXPORT_SYMBOL(rt_get_real_time);
+EXPORT_SYMBOL(rt_get_real_time_ns);
 EXPORT_SYMBOL(rt_get_base_linux_task);
 EXPORT_SYMBOL(rt_alloc_dynamic_task);
 EXPORT_SYMBOL(rt_register_watchdog);
